// @ts-ignore
import { Util } from 'bootstrap';

const NAME = 'sidemenu';
const DATA_KEY = 'ca.bs.collapse.sidemenu';
const EVENT_KEY = `.${DATA_KEY}`;
const DATA_API_KEY = '.data-api';
const JQUERY_NO_CONFLICT = $.fn[NAME];

const Default: CollapseSidebarConfig = {
    root: '.ca-root',
};

const ClassNames = {
    SHOW: 'show',
    ROOT_SHOW: 'sidebar-show',
};

const Event = {
    CLICK_DATA_API: `click${EVENT_KEY}${DATA_API_KEY}`,
    SHOW: `show${EVENT_KEY}`,
    HIDE: `hide${EVENT_KEY}`,
};

const Selector = {
    DATA_TOGGLE: '[data-toggle="sidemenu"]',
};

interface CollapseSidebarConfig {
    mode?: string;
    root?: string;
    auto?: boolean;
}

export default class CollapseSidebar {
    private readonly _$element: JQuery;
    private readonly _$target: any;
    private readonly _$root: JQuery;
    private readonly _config: CollapseSidebarConfig;

    private readonly _hideHandler: JQuery.EventHandler<HTMLElement>;

    constructor(element: HTMLElement, config: CollapseSidebarConfig) {
        this._hideHandler = this._onHide.bind(this);

        this._$element = $(element);
        this._$target = $(this._$element.attr('href'));
        this._$root = $(config.root);
        this._config = config;
        this._$target
            .collapse({ toggle: false })
            .on('show.bs.collapse', () => {
                this._$element.attr('aria-expanded', 'true');
                this._$element.trigger(Event.SHOW);
            })
            .on('hide.bs.collapse', () => {
                this._$element.attr('aria-expanded', 'false');
                this._$element.trigger(Event.HIDE);
            });
        if (this._config.auto) {
            this._$root.on('hide.ca.bs.layout', this._hideHandler);
        }
    }

    toggle() {
        if (this._$root.hasClass(ClassNames.ROOT_SHOW)) {
            this._$target.collapse('toggle');
        }
    }

    private _onHide(ev: JQueryEventObject) {
        if (ev['layout'] === 'sidebar') {
            this._$target.collapse('hide');
        }
    }

    // tslint:disable-next-line:naming-convention
    static _jQueryInterface(this: HTMLElement, config: CollapseSidebarConfig | string) {
        const $this = $(this);
        let data = $this.data(DATA_KEY) as CollapseSidebar;
        if (!data) {
            const cfg = {
                ...Default,
                ...(typeof config === 'object' && config),
            };
            data = new CollapseSidebar(this, cfg);
            $this.data(DATA_KEY, data);
        }
        data.toggle();
    }
}

$(document).on(Event.CLICK_DATA_API, Selector.DATA_TOGGLE, event => {
    if (event.currentTarget.tagName === 'A') {
        event.preventDefault();
    }
    const $trigger = $(event.currentTarget);
    const data = $trigger.data(DATA_KEY) as CollapseSidebar;
    const config = data ? 'toggle' : $trigger.data();
    CollapseSidebar._jQueryInterface.call($trigger[0], config);
});

$.fn[NAME] = CollapseSidebar._jQueryInterface;
$.fn[NAME].Constructor = CollapseSidebar;
$.fn[NAME].noConflict = function() {
    $.fn[NAME] = JQUERY_NO_CONFLICT;
    return CollapseSidebar._jQueryInterface;
};
