const path = require('path');
const webpack = require('webpack');
const ExtractTextPlugin = require('extract-text-webpack-plugin');
const CleanWebpackPlugin = require('clean-webpack-plugin');
const info = require('./package.json');

const main = path.parse(info.main);
const style = path.parse(info.style);

module.exports = [
    {
        name: 'js',
        entry: './src/scripts/boot.ts',
        output: {
            filename: main.base,
            path: path.join(__dirname, main.dir),
            library: {
                root: 'CarbonBootstrap',
                amd: 'carbon-bootstrap',
                commonjs: 'carbon-bootstrap',
            },
            libraryTarget: 'umd',
        },
        externals: {
            jquery: 'jQuery',
            bootstrap: 'bootstrap',
        },
        resolve: { extensions: ['.js', '.ts'] },
        module: {
            rules: [
                {
                    test: /\.ts$/,
                    include: path.join(__dirname, 'src', 'scripts'),
                    use: [
                        {
                            loader: 'ts-loader',
                            options: {
                                transpileOnly: true,
                            },
                        },
                    ],
                },
            ],
        },
        plugins: [
            // new webpack.optimize.UglifyJsPlugin(),
            // new webpack.ProvidePlugin({
            //     $: 'jquery',
            //     jQuery: 'jquery',
            //     'window.jQuery': 'jquery',
            //     Popper: ['popper.js', 'default'],
            // }),
            new CleanWebpackPlugin([path.join(__dirname, main.dir, main.base)]),
        ],
    },
    {
        name: 'css',
        entry: './src/styles/site.scss',
        output: {
            filename: style.base,
            path: path.join(__dirname, style.dir),
        },
        module: {
            rules: [
                {
                    test: /\.scss$/,
                    use: ExtractTextPlugin.extract({
                        use: [
                            {
                                loader: 'css-loader',
                                options: {
                                    minimize: true,
                                },
                            },
                            {
                                loader: 'sass-loader',
                            },
                        ],
                    }),
                },
            ],
        },
        plugins: [
            new CleanWebpackPlugin([path.join(__dirname, style.dir, style.base)]),
            new ExtractTextPlugin({
                filename: style.base,
            }),
        ],
    },
];
