const path = require('path');
const webpack = require('webpack');
const ExtractTextPlugin = require('extract-text-webpack-plugin');
const CleanWebpackPlugin = require('clean-webpack-plugin');

module.exports = (env) => {
    const isDev = !(env && env.prod);
    const extractCss = new ExtractTextPlugin('vendor.css');

    return [
        {
            resolve: { extensions: ['.js'] },
            entry: {
                vendor: [
                    'bootstrap',
                    'event-source-polyfill',
                    'font-awesome/css/font-awesome.css',
                    'ionicons/dist/css/ionicons.css',
                    'isomorphic-fetch',
                    'jquery',
                ],
            },
            module: {
                rules: [
                    {
                        test: /\.css(\?|$)/,
                        use: ExtractTextPlugin.extract({
                            use: isDev ? 'css-loader' : 'css-loader?minimize',
                        }),
                    },
                    {
                        test: /\.(ttf|otf|eot|svg|woff(2)?)(\?[a-z0-9]+)?$/,
                        use: 'url-loader?limit=100000',
                    },
                ],
            },
            output: {
                path: path.join(__dirname, 'assets'),
                filename: '[name].js',
                library: '[name]_[hash]',
            },
            plugins: [
                new CleanWebpackPlugin('assets'),
                extractCss,
                new webpack.ProvidePlugin({
                    $: 'jquery',
                    jQuery: 'jquery',
                    'window.jQuery': 'jquery',
                    Popper: ['popper.js', 'default'],
                }),
                new webpack.DllPlugin({
                    path: path.join(__dirname, 'assets', '[name]-manifest.json'),
                    name: '[name]_[hash]',
                }),
            ].concat(isDev ? [] : [new webpack.optimize.UglifyJsPlugin({ output: { comments: false } })]),
        },
    ];
};
